
#ifndef _BINDHELP
#define _BINDHELP

//======================  
// WideString 
//======================  

typedef void *PWideString;
typedef void *PAnsiString;
typedef char *PAnsiChar;
typedef unsigned short *PWideChar;

typedef void (*CopyUnicodeToPWideString)(const QChar *unicode, PWideString ps, int len);
typedef PWideChar (*UnicodeOfPWideString)(PWideString ps);
typedef int (*LengthOfPWideString)(PWideString ps);
typedef void (*InitializePWideString)(PWideString &ps);
typedef void (*FinalizePWideString)(PWideString &ps);

CopyUnicodeToPWideString copyUnicodeToPWideString;
UnicodeOfPWideString unicodeOfPWideString;
LengthOfPWideString lengthOfPWideString;
InitializePWideString initPWideString;
FinalizePWideString finalPWideString;

inline void copyQStringToPWideString(const QString &qs, PWideString ps)
{
if (qs!=0 && ps) copyUnicodeToPWideString(qs.unicode(), ps, qs.length());
}


inline void copyPWideStringToQString(PWideString ps, QString &qs)
{
  qs.setUtf16((ushort *)unicodeOfPWideString(ps),
    lengthOfPWideString(ps));
}

inline void initializePWideString(PAnsiString &pws)
{
  initPWideString(pws);
}

inline void finalizePWideString(PWideString &pws)
{
  finalPWideString(pws);
}

C_EXPORT void initPWideStrings(CopyUnicodeToPWideString cutps,
  UnicodeOfPWideString uops, LengthOfPWideString lops,
  InitializePWideString ip, FinalizePWideString fp);


  
//======================  
// QRect
// Pascal  : left,top,right,bottom
// Qt        : left,top,right,bottom aka x1,y1,x2,y2
// Qt MAC: top,left,bottom,right aka y1,x1,y2,x2 
//======================

typedef void *PRect;

inline void copyQRectToPRect(const QRect &qr, PRect pr)
{
#if defined DARWIN
  ((QRect *)pr)->setLeft(qr.top());
  ((QRect *)pr)->setTop(qr.left());
  ((QRect *)pr)->setRight(qr.bottom()+1);
  ((QRect *)pr)->setBottom(qr.right()+1);
#else
  ((QRect *)pr)->setLeft(qr.left());
  ((QRect *)pr)->setTop(qr.top());
  ((QRect *)pr)->setRight(qr.right()+1);
  ((QRect *)pr)->setBottom(qr.bottom()+1);
#endif  
}

inline void copyPRectToQRect(PRect pr, QRect &qr)
{
#if defined DARWIN
  qr.setLeft(((QRect *)pr)->top());
  qr.setTop(((QRect *)pr)->left());
  qr.setRight(((QRect *)pr)->bottom()-1);
  qr.setBottom(((QRect *)pr)->right()-1);
#else
  qr.setLeft(((QRect *)pr)->left());
  qr.setTop(((QRect *)pr)->top());
  qr.setRight(((QRect *)pr)->right()-1);
  qr.setBottom(((QRect *)pr)->bottom()-1);
#endif  
}

typedef struct {
  int x;
  int y;
} TPoint;
typedef TPoint *PPoint;

inline void copyQPointToPPoint(const QPoint &qp, PPoint pp)
{
#if defined DARWIN
  ((QPoint *)pp)->setX(qp.y());
  ((QPoint *)pp)->setY(qp.x());
#else
  *(QPoint *)pp = qp;
#endif  
}

inline void copyPPointToQPoint(PPoint pp, QPoint &qp)
{
#if defined DARWIN
  qp.setX(((QPoint *)pp)->y());
  qp.setY(((QPoint *)pp)->x());
#else
  qp = *(QPoint *)pp;
#endif  
}

//======================  
// QList<int>
//======================  

typedef void *PIntArray;
typedef void *(*GetIntsPtr)(PIntArray pi);
typedef int (*GetIntsLength)(PIntArray pi);
typedef void (*SetIntsLength)(PIntArray pi, int len);
GetIntsPtr getIntsPtr;
GetIntsLength getIntsLength;
SetIntsLength setIntsLength;


template <typename T>
Q_OUTOFLINE_TEMPLATE void copyQListTemplateToPInts(QList<T> &qi, PIntArray pi)
{
  int len = qi.count();
  PTRINT *ints;
  setIntsLength(pi, len);
  ints = (PTRINT *)getIntsPtr(pi);
  for (int i = 0; i < len; i++)
    ints[i] = (PTRINT)qi[i];
}



template <typename T>
Q_OUTOFLINE_TEMPLATE void copyPIntsToQListTemplate(PIntArray pi,QList<T> &qi)
{
  int len = getIntsLength(pi);
  PTRINT *ints = (PTRINT *)getIntsPtr(pi);
  qi.clear();
  for (int i = 0; i < len; i++)
    qi.append((T)ints[i]);
}






C_EXPORT void initializePIntArray(GetIntsPtr gip, GetIntsLength gil, SetIntsLength sil);


#endif
